#ifndef COREOGRAPHY_INTERPOLATION_HPP
#define COREOGRAPHY_INTERPOLATION_HPP

#include <vector>
#include <string>

class Interpolation;

Interpolation* get_interpolation(char method, std::vector<std::string> arguments);

class Interpolation{
protected:
    Interpolation() = default;
public:
    virtual ~Interpolation() = default;
    /**
     * Interpolates between two points.
     * 
     * @param before : The value of the element before the current moment.
     * @param after  : The value of the element after the current moment.
     * @param middle : A value in the range [0, 1] that shows how close
     *                 to before or after values the .
     * 
     * @return : The interpolated value.
     */
    virtual float interpolate(float before, float after, float middle) = 0;
    virtual Interpolation* clone() = 0;
};

class InterpolationLinear: public Interpolation{
public:
    InterpolationLinear(std::vector<std::string>);
    InterpolationLinear();
    float interpolate(float, float, float);
    Interpolation* clone();
};

class InterpolationWeighted: public Interpolation{
    float weight_in;
    float weight_out;
public:
    InterpolationWeighted(std::vector<std::string>);
    InterpolationWeighted(float, float);
    float interpolate(float, float, float);
    Interpolation* clone();
};

class InterpolationStep: public Interpolation{
    float cutoff;
public:
    InterpolationStep(std::vector<std::string>);
    InterpolationStep(float);
    float interpolate(float, float, float);
    Interpolation* clone();
};

#endif
