#include "actions.hpp"

Action::Action(){}
Action::~Action(){}

ActionConfigReload::ActionConfigReload(): Action(){}
ActionConfigReload::~ActionConfigReload(){}
void ActionConfigReload::do_action(
    MotorControl*& motor_control,
    Coreographer*& coreographer,
    int& rhythm,
    std::function<void ()>& update_variables
){
    update_variables();
};

ActionChangeRhythm::ActionChangeRhythm(int nr): Action(){this->new_rhythm = nr;}
ActionChangeRhythm::~ActionChangeRhythm(){}
void ActionChangeRhythm::do_action(
    MotorControl*& motor_control,
    Coreographer*& coreographer,
    int& rhythm,
    std::function<void ()>& update_variables
){
    rhythm = this->new_rhythm;
};

ActionDisengage::ActionDisengage(): Action(){}
ActionDisengage::~ActionDisengage(){}
void ActionDisengage::do_action(
    MotorControl*& motor_control,
    Coreographer*& coreographer,
    int& rhythm,
    std::function<void ()>& update_variables
){
    motor_control->left_stepper->disengage();
    motor_control->right_stepper->disengage();
};

ActionReengage::ActionReengage(): Action(){}
ActionReengage::~ActionReengage(){}
void ActionReengage::do_action(
    MotorControl*& motor_control,
    Coreographer*& coreographer,
    int& rhythm,
    std::function<void ()>& update_variables
){
    motor_control->left_stepper->reengage();
    motor_control->right_stepper->reengage();
};

ActionSetCoreographer::ActionSetCoreographer(Coreographer* nc): Action(){this->new_coreographer = nc;}
ActionSetCoreographer::~ActionSetCoreographer(){}
void ActionSetCoreographer::do_action(
    MotorControl*& motor_control,
    Coreographer*& coreographer,
    int& rhythm,
    std::function<void ()>& update_variables
){
    if(this->new_coreographer == nullptr) return;
    Coreographer* old_coreographer = coreographer;
    coreographer = this->new_coreographer;
    motor_control->left_stepper->ensure_engagement();
    motor_control->right_stepper->ensure_engagement();
    delete old_coreographer;
    this->new_coreographer = nullptr;
};