#include "bootsequence.hpp"

BootSequence::BootSequence(LED* led){
    this->bootpin = new BootPin();
    this->bootpin->bind();
    this->led = led;
}

void BootSequence::choose_from_pin(){
    int seq = 0;
    for(int i=0; this->bootpin->read(); i++){
        seq = i/20;
        switch(seq){
            case 1:
            this->led->r->write(false);
            this->led->g->write(false);
            this->led->b->write(i%2);
            break;
            case 2:
            this->led->r->write(false);
            this->led->g->write(i%2);
            this->led->b->write(false);
            break;
            case 3:
            this->led->r->write(i%2);
            this->led->g->write(false);
            this->led->b->write(false);
            break;
            default:
            this->led->r->write(i%2);
            this->led->g->write(i%2);
            this->led->b->write(i%2);
            break;
        }
        usleep(1000L*100); // 100ms
    }
    this->led->r->write(false);
    this->led->g->write(false);
    this->led->b->write(false);
    if (seq>0 && seq <=3)
        this->pending_action = seq;
}

void BootSequence::choose_from_storage(VFS* vfs){
    auto* action = vfs->nvs->get_string("bootaction");
    if(action!=nullptr){
        this->pending_action = atoi(action->array);
        delete action;
    }
}

void BootSequence::remove_from_storage(VFS* vfs){
    vfs->nvs->set_string("bootaction", "0");
}

bool BootSequence::has_pending_action(){
    return this->pending_action != 0;
}

void BootSequence::do_action(VFS* vfs){
    if (vfs == nullptr)
        vfs = new VFS();
    switch(this->pending_action){
        case 1:
        vfs->nvs->rm("bluetooth_mac");
        this->reboot();
        break; ////////////////////
        case 2:
        nvs_flash_erase();
        this->reboot();
        break; ////////////////////
        case 3:
        nvs_flash_erase();
        esp_spiffs_format("spiffs");
        this->reboot();
        break; ////////////////////
        default:
        this->reboot();
        break;
    }
}

void BootSequence::reboot(){
    esp_restart();
}
