#include "smartstepper.hpp"

SmartStepper::SmartStepper(StepperMotor* sm, StepperSensors* ss){
    this->stepper_motor = sm;
    this->stepper_sensors = ss;
}

bool SmartStepper::get_engaged(){ return this->engaged; }
bool SmartStepper::get_engaging(){ return this->engaging; }

int SmartStepper::get_target(){ return this->stepper_motor->get_target(); }
void SmartStepper::set_target(int tgt){ this->stepper_motor->set_target(tgt); }
void SmartStepper::disengage(){
    this->engaged = false;
    this->engaging = false;
    this->stepper_motor->disable_steppers();
}

void SmartStepper::pursue_calibration(){
    if (this->stepper_sensors->is_in_equilibrium()){
        // Calibration done -> set engaged
        this->engaging = false;
        this->engaged = true;
        this->stepper_motor->reset_state();
    } else if (this->stepper_sensors->get_bottom_state()) {
        // Top sensor activated -> rotate
        this->stepper_motor->rotate();
    } else if (this->stepper_sensors->get_top_state()) {
        // Bottom sensor activated -> rotate the other way
        this->stepper_motor->rotate_inverse();
    }
}

void SmartStepper::pursue_target(){
    if (!this->engaged && !this->engaging){
        // disengaged and not engaging -> ignore
    }else if (this->engaging && !this->engaged) {
        // disengaged and engaging -> calibrate
        this->pursue_calibration();
    }else{
        // engaged -> pursue defined target
        this->stepper_motor->pursue_target();
    }
}

bool SmartStepper::is_target_reached(){
    if (!this->engaged)
        return false;
    else
        return this->stepper_motor->is_target_reached();
}

void SmartStepper::reengage(){
    this->disengage();
    this->engaging = true;
    this->stepper_motor->enable_steppers();
}

void SmartStepper::ensure_engagement(){
    if(!this->engaged && !this->engaging){
        this->reengage();
    }
}